
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <compiler_defs.h>
#include <SI_C8051F380_Register_Enums.h>                // SFR declarations
#include <stdio.h>
#include <Ethernet/socket.h>

//////////////////////////////////////
// Define WIZCHIP_RESET_PIN : P1_0) //
SBIT(WIZCHIP_RESET_PIN, SFR_P1, 0);
//////////////////////////////////////


/////////////////////////////////////////
// SOCKET NUMBER DEFINION for Examples //
/////////////////////////////////////////
#define SOCK_TCPS        0
#define SOCK_UDPS        1

////////////////////////////////////////////////
// Shared Buffer Definition for LOOPBACK TEST //
////////////////////////////////////////////////
#define DATA_BUF_SIZE   2048
uint8_t gDATABUF[DATA_BUF_SIZE];


wiz_NetInfo xdata gWIZNETINFO;/* = { {0x10, 0x11, 0x12,0x13, 0x14, 0x15},
                            {192, 168, 1, 119},
                            {255,255,255,0},
                            {192, 168, 1, 1},
                            {0,0,0,0},
                            NETINFO_STATIC }; */

void Timer_Init()
{
    TCON      = 0x40;
    TMOD      = 0x20;
    CKCON     = 0x08;
    TH1       = 0x30;
//    TCON_TR1  = 1;
}

void UART_Init()
{
    SCON0     = 0x10;
    SCON0_TI = 1;
}

void SPI_Init()
{
    SPI0CFG   = 0x40;
    SPI0CN    = 0x0D;
    SPI0CKR   = 0x01;
}

void Port_IO_Init()
{
    // P0.0  -  SCK  (SPI0), Push-Pull,  Digital
    // P0.1  -  MISO (SPI0), Open-Drain, Digital
    // P0.2  -  MOSI (SPI0), Push-Pull,  Digital
    // P0.3  -  NSS  (SPI0), Push-Pull,  Digital
    // P0.4  -  TX0 (UART0), Push-Pull,  Digital
    // P0.5  -  RX0 (UART0), Open-Drain, Digital
    // P0.6  -  Unassigned,  Open-Drain, Digital
    // P0.7  -  Unassigned,  Open-Drain, Digital

    // P1.0  -  Unassigned,  Push-Pull,  Digital
    // P1.1  -  Unassigned,  Open-Drain, Digital
    // P1.2  -  Unassigned,  Open-Drain, Digital
    // P1.3  -  Unassigned,  Open-Drain, Digital
    // P1.4  -  Unassigned,  Open-Drain, Digital
    // P1.5  -  Unassigned,  Open-Drain, Digital
    // P1.6  -  Unassigned,  Open-Drain, Digital
    // P1.7  -  Unassigned,  Open-Drain, Digital

    // P2.0  -  Unassigned,  Open-Drain, Digital
    // P2.1  -  Unassigned,  Open-Drain, Digital
    // P2.2  -  Unassigned,  Open-Drain, Digital
    // P2.3  -  Unassigned,  Open-Drain, Digital
    // P2.4  -  Unassigned,  Open-Drain, Digital
    // P2.5  -  Unassigned,  Open-Drain, Digital
    // P2.6  -  Unassigned,  Open-Drain, Digital
    // P2.7  -  Unassigned,  Open-Drain, Digital

    // P3.0  -  Unassigned,  Open-Drain, Digital
    // P3.1  -  Unassigned,  Open-Drain, Digital
    // P3.2  -  Unassigned,  Open-Drain, Digital
    // P3.3  -  Unassigned,  Open-Drain, Digital
    // P3.4  -  Unassigned,  Open-Drain, Digital
    // P3.5  -  Unassigned,  Open-Drain, Digital
    // P3.6  -  Unassigned,  Open-Drain, Digital
    // P3.7  -  Unassigned,  Open-Drain, Digital

    P0MDOUT   = 0x1D;
    P1MDOUT   = 0x01;
    XBR0      = 0x03;
    XBR1      = 0x40;
}

void Oscillator_Init()
{
    FLSCL     = 0x90;
    CLKSEL    = 0x03;
    OSCICN    = 0xC3;
}

// Initialization function for device,
// Call Init_Device() from your main program
void Init_Device(void)
{
	Timer_Init();
    UART_Init();
    SPI_Init();
    Port_IO_Init();
	Oscillator_Init();
}

////////////////////////////////////////
// Call-back function for WIZnet Chip //
////////////////////////////////////////
void wizchip_select(void)
{
	SPI0CN_NSSMD0   = 0;
}

void wizchip_deselect(void)
{
	SPI0CN_NSSMD0   = 1;
}

void SPI0_WriteByte(uint8_t dat)
{
	SPI0DAT  = dat;
	while (!SPI0CN_SPIF);
	SPI0CN_SPIF     = 0;
}

uint8_t SPI0_ReadByte(void)
{
	SPI0DAT  = 0;
	while (!SPI0CN_SPIF);
	SPI0CN_SPIF     = 0;
	return SPI0DAT;
}


///////////////////////////
// Reset for WIZnet Chip //
///////////////////////////
void WIZChip_HW_Reset(void)
{
	volatile uint16_t i;
	WIZCHIP_RESET_PIN = 0;
	for(i = 0; i < 0xFFFF; i++); // 20ms@48MHz
	WIZCHIP_RESET_PIN = 1;
	for(i = 0; i < 0xFFFF; i++); // 20ms@48MHz
	for(i = 0; i < 0xFFFF; i++); // 20ms@48MHz

}

/////////////////////////////////////////////////////////////
// Intialize the network information to be used in WIZCHIP //
/////////////////////////////////////////////////////////////
void network_init(void)
{
   uint8_t tmpstr[6];
	ctlnetwork(CN_SET_NETINFO, (void*)&gWIZNETINFO);
	ctlnetwork(CN_GET_NETINFO, (void*)&gWIZNETINFO);

	// Display Network Information
	ctlwizchip(CW_GET_ID,(void*)tmpstr);
	printf("\r\n=== %s NET CONF ===\r\n",(char*)tmpstr);
	printf("MAC: %02bX:%02bX:%02bX:%02bX:%02bX:%02bX\r\n",gWIZNETINFO.mac[0],gWIZNETINFO.mac[1],gWIZNETINFO.mac[2],
		  gWIZNETINFO.mac[3],gWIZNETINFO.mac[4],gWIZNETINFO.mac[5]);
	printf("SIP: %bu.%bu.%bu.%bu\r\n", gWIZNETINFO.ip[0],gWIZNETINFO.ip[1],gWIZNETINFO.ip[2],gWIZNETINFO.ip[3]);
	printf("GAR: %bu.%bu.%bu.%bu\r\n", gWIZNETINFO.gw[0],gWIZNETINFO.gw[1],gWIZNETINFO.gw[2],gWIZNETINFO.gw[3]);
	printf("SUB: %bu.%bu.%bu.%bu\r\n", gWIZNETINFO.sn[0],gWIZNETINFO.sn[1],gWIZNETINFO.sn[2],gWIZNETINFO.sn[3]);
	printf("DNS: %bu.%bu.%bu.%bu\r\n", gWIZNETINFO.dns[0],gWIZNETINFO.dns[1],gWIZNETINFO.dns[2],gWIZNETINFO.dns[3]);
	printf("======================\r\n");
}
/////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Loopback Test Example Code using ioLibrary_BSD			 //
///////////////////////////////////////////////////////////////
int32_t loopback_tcps(uint8_t sn, uint8_t* buf, uint16_t port)
{
   int32_t ret;
   uint16_t size = 0, sentsize=0;
   switch(getSn_SR(sn))
   {
      case SOCK_ESTABLISHED :
         if(getSn_IR(sn) & Sn_IR_CON)
         {
            printf("%bu:Connected\r\n",sn);
            setSn_IR(sn,Sn_IR_CON);
         }
         if((size = getSn_RX_RSR(sn)) > 0)
         {
            if(size > DATA_BUF_SIZE) size = DATA_BUF_SIZE;
            ret = recv(sn,buf,size);
            if(ret <= 0) return ret;
            sentsize = 0;
            while(size != sentsize)
            {
               ret = send(sn,buf+sentsize,size-sentsize);
               if(ret < 0)
               {
                  close(sn);
                  return ret;
               }
               sentsize += ret; // Don't care SOCKERR_BUSY, because it is zero.
            }
         }
         break;
      case SOCK_CLOSE_WAIT :
         printf("%bu:CloseWait\r\n",sn);
         if((ret=disconnect(sn)) != SOCK_OK) return ret;
         printf("%bu:Closed\r\n",sn);
         break;
      case SOCK_INIT :
    	  printf("%bu:Listen, port [%u]\r\n",sn, port);
         if( (ret = listen(sn)) != SOCK_OK) return ret;
         break;
      case SOCK_CLOSED:
         printf("%bu:LBTStart\r\n",sn);
         if((ret=socket(sn,Sn_MR_TCP,port,0x00)) != sn)
            return ret;
         printf("%bu:Opened\r\n",sn);
         break;
      default:
         break;
   }
   return 1;
}

int32_t loopback_udps(uint8_t sn, uint8_t* buf, uint16_t port)
{
   int32_t  ret;
   uint16_t size, sentsize;
   uint8_t  destip[4];
   uint16_t destport;
   //uint8_t  packinfo = 0;
   switch(getSn_SR(sn))
   {
      case SOCK_UDP :
         if((size = getSn_RX_RSR(sn)) > 0)
         {
            if(size > DATA_BUF_SIZE) size = DATA_BUF_SIZE;
            ret = recvfrom(sn,buf,size,destip,(uint16_t*)&destport);
            if(ret <= 0)
            {
               printf("%bu: recvfrom error. %ld\r\n",sn,ret);
               return ret;
            }
            size = (uint16_t) ret;
            sentsize = 0;
            while(sentsize != size)
            {
               ret = sendto(sn,buf+sentsize,size-sentsize,destip,destport);
               if(ret < 0)
               {
                  printf("%bu: sendto error. %ld\r\n",sn,ret);
                  return ret;
               }
               sentsize += ret; // Don't care SOCKERR_BUSY, because it is zero.
            }
         }
         break;
      case SOCK_CLOSED:
         printf("%bu:LBUStart\r\n",sn);
         if((ret=socket(sn,Sn_MR_UDP,port,0x00)) != sn)
            return ret;
         printf("%bu:Opened, port [%u]\r\n",sn, port);
         break;
      default :
         break;
   }
   return 1;
}
//////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
//
// Note: the software watchdog timer is not disabled by default in this
// example, so a long-running program will reset periodically unless
// the timer is disabled or your program periodically writes to it.
//
// Review the "Watchdog Timer" section under the part family's datasheet
// for details. To find the datasheet, select your part in the
// Simplicity Launcher and click on "Data Sheet".
//
//-----------------------------------------------------------------------------
int main (void)
{
	uint8_t tmp;
	uint8_t xdata memsize[2][8] = {{2,2,2,2,2,2,2,2},{2,2,2,2,2,2,2,2}};

	gWIZNETINFO.mac[0] = 0x10;
	gWIZNETINFO.mac[1] = 0x11;
	gWIZNETINFO.mac[2] = 0x12;
	gWIZNETINFO.mac[3] = 0x13;
	gWIZNETINFO.mac[4] = 0x14;
	gWIZNETINFO.mac[5] = 0x15;
	gWIZNETINFO.ip[0] = 192;
	gWIZNETINFO.ip[1] = 168;
	gWIZNETINFO.ip[2] = 1;
	gWIZNETINFO.ip[3] = 119;
	gWIZNETINFO.sn[0] = 255;
	gWIZNETINFO.sn[1] = 255;
	gWIZNETINFO.sn[2] = 255;
	gWIZNETINFO.sn[3] = 0;
	gWIZNETINFO.gw[0] = 192;
	gWIZNETINFO.gw[1] = 168;
	gWIZNETINFO.gw[2] = 1;
	gWIZNETINFO.gw[3] = 1;
	gWIZNETINFO.dns[0] = 0;
	gWIZNETINFO.dns[1] = 0;
	gWIZNETINFO.dns[2] = 0;
	gWIZNETINFO.dns[3] = 0;
	gWIZNETINFO.dhcp = NETINFO_STATIC;

	Init_Device();

	printf("Welcome to W5500 ioLibrary Porting Guide for 8051 Users\r\n");

	WIZChip_HW_Reset();

	printf("WIZCHP TEST : Registeration Callback func\r\n");
	//reg_wizchip_cris_cbfucn(0,0); // No-use in example
	reg_wizchip_cs_cbfunc(wizchip_select,wizchip_deselect);
	reg_wizchip_spi_cbfunc(SPI0_ReadByte,SPI0_WriteByte);


	/* W5500 Access Test */
	// This code should be commented after testing W5500 access //
	/*
	if(getVERSIONR() != 0x04)
	{
		printf("ACCESS ERR: VERSIONR != 0x04, Read value=%02bx\r\n", getVERSIONR());
		while(1);
	}

	if(getRCR() != 0x08)
	{
		printf("ACCESS ERR: RCR != 0x08, Read value=%02bx\r\n",getRCR());
		while(1);
	}
	if(getRTR() != 0x07D0)
	{
		printf("ACCESS ERR: RTR != 0x07D0, Read value=%04x\r\n", getRTR());
		while(1);
	}

	setRCR(0xA5);
	setRTR(0x5A5A);
	if(getRCR() != 0xA5)
	{
		printf("ACCESS ERR: RCR != 0xA5, Read value=%02bx\r\n",getRCR());
		while(1);
	}
	if(getRTR() != 0x5A5A)
	{
		printf("ACCESS ERR: RTR != 0x5A5A, Read value=%04x\r\n", getRTR());
		while(1);
	}
	/////////////////////////////////////////////////////////////////////
    */

	/* WIZCHIP SOCKET Buffer initialize */

	if(ctlwizchip(CW_INIT_WIZCHIP,(void*)memsize) == -1)
	{
	   printf("WIZCHIP Initialized fail.\r\n");
	   while(1);
	}
	/* PHY link status check */
	printf("Link Check...");
	do
	{
	   if(ctlwizchip(CW_GET_PHYLINK, (void*)&tmp) == -1)
		  printf("Unknown PHY Link stauts.\r\n");
	}while(tmp == PHY_LINK_OFF);
	printf("OK\r\n");

	/* Network initialization */
	network_init();

	/*******************************/
	/* WIZnet W5500 Code Examples  */
	/* TCPS/UDPS Loopback test     */
	/*******************************/
    /* Main loop */
    while(1)
	{
    	uint32_t ret = 0;
    	/* Loopback Test */
    	// TCP server loopback test
    	if( (ret = loopback_tcps(SOCK_TCPS, gDATABUF, 5000)) < 0) {
			printf("SOCKET ERROR : %ld\r\n", ret);
		}

    	// UDP server loopback test
		if( (ret = loopback_udps(SOCK_UDPS, gDATABUF, 3000)) < 0) {
			printf("SOCKET ERROR : %ld\r\n", ret);
		}
	} // end of Main loop
	// NOTREACHED

	return 0;
}
